@@ -1,20 +1,22 @@
 import os
 import typing
-from datetime import datetime
-
 from datetime import datetime
 from importlib import import_module
 from typing import Dict, Callable, Optional, List, Tuple
-from .util import temp_syspath
+
 from twitchbot.database import CustomCommand
 from twitchbot.message import Message
 from .config import cfg
 from .enums import CommandContext
 from .util import get_py_files, get_file_name
+from .util import temp_syspath
 
 if typing.TYPE_CHECKING:
     from .modloader import Mod
 
+DEFAULT_COOLDOWN_BYPASS = 'bypass_cooldown'
+DEFAULT_COOLDOWN = 0
+
 __all__ = (
     'Command', 'commands', 'command_exist', 'load_commands_from_directory', 'DummyCommand', 'CustomCommandAction',
     'ModCommand', 'SubCommand', 'get_command', 'CUSTOM_COMMAND_PLACEHOLDERS', 'command_last_execute',
@@ -25,8 +27,8 @@ __all__ = (
 class Command:
     def __init__(self, name: str, prefix: str = None, func: Callable = None, global_command: bool = True,
                  context: CommandContext = CommandContext.CHANNEL, permission: str = None, syntax: str = None,
-                 help: str = None, aliases: List[str] = None, cooldown: int = 0,
-                 cooldown_bypass: str = 'bypass_cooldown'):
+                 help: str = None, aliases: List[str] = None, cooldown: int = DEFAULT_COOLDOWN,
+                 cooldown_bypass: str = DEFAULT_COOLDOWN_BYPASS):
         """
         :param cooldown: time between when this command when can be run, 0 means the command be run without any delay and is default value
         :param syntax: help message for how to use the command, <> is required, () is optional
@@ -94,9 +96,9 @@ class Command:
 
 class SubCommand(Command):
     def __init__(self, parent: Command, name: str, func: Callable = None, permission: str = None, syntax: str = None,
-                 help: str = None):
+                 help: str = None, cooldown: int = DEFAULT_COOLDOWN, cooldown_bypass: str = DEFAULT_COOLDOWN_BYPASS):
         super().__init__(name=name, prefix='', func=func, permission=permission, syntax=syntax, help=help,
-                         global_command=False)
+                         global_command=False, cooldown=cooldown, cooldown_bypass=cooldown_bypass)
 
         self.parent: Command = parent
         self.parent.sub_cmds[self.name] = self
@@ -150,6 +152,7 @@ class CustomCommandAction(Command):
     def __init__(self, cmd):
         super().__init__(cmd.name, prefix='', func=self.execute, global_command=False)
         self.cmd: CustomCommand = cmd
+        self.cooldown = 0
 
     async def execute(self, msg: Message):
         resp = self.cmd.response
@@ -164,19 +167,18 @@ class CustomCommandAction(Command):
 class ModCommand(Command):
     def __init__(self, mod_name: str, name: str, prefix: str = None, func: Callable = None, global_command: bool = True,
                  context: CommandContext = CommandContext.CHANNEL, permission: str = None, syntax: str = None,
-                 help: str = None):
+                 help: str = None, cooldown: int = DEFAULT_COOLDOWN, cooldown_bypass: str = DEFAULT_COOLDOWN_BYPASS):
         super().__init__(name=name, prefix=prefix, func=func, global_command=global_command, context=context,
-                         permission=permission, syntax=syntax, help=help)
+                         permission=permission, syntax=syntax, help=help, cooldown=cooldown,
+                         cooldown_bypass=cooldown_bypass)
         self.mod_name = mod_name
-        self.mod: 'Mod' = None
+
+    @property
+    def mod(self):
+        from .modloader import mods
+        return mods[self.mod_name]
 
     async def execute(self, msg: Message):
-        # circular dependency hack
-        from .modloader import mods
-
-        if self.mod is None:
-            self.mod = mods[self.mod_name]
-
         func, args = self._get_cmd_func(msg.parts[1:])
         if 'self' in func.__code__.co_varnames:
             await func(self.mod, msg, *args)
@@ -193,6 +195,8 @@ def _create_cooldown_key(channel: str, cmd: str) -> Tuple[str, str]:
 
 
 def is_command_off_cooldown(channel: str, cmd: str, cooldown: int = None) -> bool:
+    if not command_exist(cmd):
+        return True
     return get_time_since_execute(channel, cmd) >= (cooldown or get_command(cmd).cooldown)
 
 
